#include "Q7BEmailWrapper.h"


/*
***************************************************************************
Copyright (C) 2000-2010 KBasic Software: www.kbasic.com. All rights reserved.
Source codes are copyrighted by Bernd Noetscher 2000-2010.

You may use this source codes under the terms of the GNU Public License (Version 3) as published 
by the Free Software Foundation. 

If you are interested in using the source codes for other licenses or 
commercial development, you must buy commercial licenses from KBasic Software.
***************************************************************************
*/


void Q7BEmailWrapper::login( QString smtpSrvrHst ,
                              int smtpPrt ,
                              QString pop3SrvrHst ,
                              int pop3Prt ,
                              bool scrd ,
                              QString username ,
                              QString password)
{
    smtp = new QxtSmtp( this ) ;

    smtp->setUsername( username.toAscii() ) ;
    smtp->setPassword( password.toAscii() ) ;
    smtpServerHost = smtpSrvrHst ;
    smtpPort = smtpPrt ;

    pop3ServerHost = pop3SrvrHst ;
    pop3Port = pop3Prt ;
    secured = scrd ;

    connect( smtp , SIGNAL( connected() ) , SLOT( smtpConnected() ) ) ;
    connect( smtp , SIGNAL( encrypted() ) , SLOT( smtpEncrypted() ) ) ;
    connect( smtp , SIGNAL( authenticated() ) , SLOT( smtpAuthenticated() ) ) ;
    connect( smtp , SIGNAL( authenticationFailed( QByteArray ) ) , SLOT( smtpError( QByteArray ) ) ) ;
    connect( smtp , SIGNAL( connectionFailed( QByteArray ) ) , SLOT( smtpError( QByteArray ) ) ) ;
    connect( smtp , SIGNAL( encryptionFailed( QByteArray ) ) , SLOT( smtpError( QByteArray ) ) ) ;
    connect( smtp , SIGNAL(mailFailed(int,int,QByteArray)) , SLOT(smtpMailFailed(int,int,QByteArray))) ;
    connect( smtp , SIGNAL(recipientRejected(int,QString,QByteArray)) , SLOT(smtpRecipientRejected(int,QString,QByteArray))) ;
    connect( smtp , SIGNAL(senderRejected(int,QString,QByteArray)) , SLOT(smtpSenderRejected(int,QString,QByteArray))) ;
    connect( smtp , SIGNAL(mailSent(int)) , SLOT(smtpMailSent(int)) ) ;
    connect( smtp , SIGNAL(finished()) , SLOT(smtpFinished()) ) ;

    pop3 = new QxtPop3( this ) ;
    pop3->setUsername( username.toAscii() ) ;
    pop3->setPassword( password.toAscii() ) ;
    pop3->setStartTlsDisabled( true ) ;

    connect( pop3 , SIGNAL( connected() ) , SLOT( pop3Connected() ) ) ;
    connect( pop3 , SIGNAL( encrypted() ) , SLOT( pop3Encrypted() ) ) ;
    connect( pop3 , SIGNAL( authenticated() ) , SLOT( pop3Authenticated() ) ) ;
    connect( pop3 , SIGNAL( authenticationFailed( QByteArray ) ) , SLOT( pop3Error( QByteArray ) ) ) ;
    connect( pop3 , SIGNAL( connectionFailed( QByteArray ) ) , SLOT( pop3Error( QByteArray ) ) ) ;
    connect( pop3 , SIGNAL( encryptionFailed( QByteArray ) ) , SLOT( pop3Error( QByteArray ) ) ) ;
}

void Q7BEmailWrapper::smtpConnected()
{
    LastError = "" ;
    qDebug() << "smtp connected" ;
}

void Q7BEmailWrapper::smtpEncrypted()
{
    LastError = "" ;
    qDebug() << "smtp encrypted" ;
}

void Q7BEmailWrapper::smtpAuthenticated()
{
    LastError = "" ;
    qDebug() << "smtp authenticated" ;
}

void Q7BEmailWrapper::smtpError( QByteArray er )
{
    LastError = er ;
    qDebug() << LastError ;
    emit stop() ;
}

void Q7BEmailWrapper::smtpMailFailed ( int , int , const QByteArray & msg )
{
    LastError = "smtp mail failed: " + msg ;
    qDebug() << LastError ;
    emit stop() ;
}

void Q7BEmailWrapper::smtpMailSent ( int  )
{
    LastError = "" ;
    qDebug() << "smtp mail sent :)" ;
}

void Q7BEmailWrapper::smtpRecipientRejected ( int , const QString & , const QByteArray & msg )
{
    LastError = "smtp recipient rejected: " + msg ;
    qDebug() << LastError ;
    emit stop() ;
}

void Q7BEmailWrapper::smtpFinished ()
{

}

void Q7BEmailWrapper::smtpSenderRejected ( int , const QString & , const QByteArray & msg )
{
    LastError = "smtp sender rejected" + msg ;
    qDebug() << LastError ;
    emit stop() ;
}

QString Q7BEmailWrapper::sendMail( QString frm ,
                                 QStringList toRecipients ,
                                 QStringList toRecipientsCC ,
                                 QStringList toRecipientsBCC ,
                                 QString subject ,
                                 QString body ,
                                 QStringList atttachmenFilestList )
{
    if( secured )
        smtp->connectToSecureHost( smtpServerHost , smtpPort ) ;
    else
        smtp->connectToHost( smtpServerHost , smtpPort ) ;
    QEventLoop eventLoop ;
    connect( smtp , SIGNAL( authenticated( ) ) , & eventLoop , SLOT( quit() ) ) ;
    connect( this , SIGNAL(stop()) ,& eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;
    if( LastError != "" ) return LastError ;

    QxtMailMessage * m = new QxtMailMessage ;

    foreach( QString cur , toRecipients )
        m->addRecipient( cur , QxtMailMessage::To ) ;
    foreach( QString cur , toRecipientsCC )
        m->addRecipient( cur , QxtMailMessage::Cc ) ;
    foreach( QString cur , toRecipientsBCC )
        m->addRecipient( cur , QxtMailMessage::Bcc ) ;
    m->setSender( frm ) ;
    m->setSubject( subject ) ;
    m->setBody( body ) ;

    QList< QxtMailAttachment * > attList ;
    QList< QFile * > filesList ;

    foreach( QString cur , atttachmenFilestList )
    {
        QFile * attFile = new QFile( cur ) ;
        filesList.append( attFile ) ;
        if( ! attFile->open( QIODevice::ReadOnly ) )
            return "Cannot open attachment file" ;
        QxtMailAttachment * att  = new QxtMailAttachment ( attFile ) ;
        attList.append( att ) ;
        m->addAttachment( cur , * att ) ;
    }

    smtp->send( * m ) ;

    connect( smtp , SIGNAL( mailSent(int ) ) , & eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;

    foreach( QFile * cur , filesList )
        delete cur ;
    foreach( QxtMailAttachment * cur , attList )
        delete cur ;
    delete m ;

    smtp->disconnectFromHost() ;
    connect( this , SIGNAL(stop()) ,& eventLoop , SLOT( quit() ) ) ;
    connect( smtp , SIGNAL(disconnected()) , & eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;
    if( LastError != "" ) return LastError ;

    return LastError ;
}

void Q7BEmailWrapper::pop3Connected()
{
    LastError = "" ;
    qDebug() << "pop3 connected" ;
}
void Q7BEmailWrapper::pop3Encrypted()
{
    LastError = "" ;
    qDebug() << "pop3 encrypted" ;
}
void Q7BEmailWrapper::pop3Authenticated()
{
    LastError = "" ;
    qDebug() << "pop3 authenticated" ;
}
void Q7BEmailWrapper::pop3Error( QByteArray er )
{
    LastError = er ;
    qDebug() << LastError ;
    emit stop() ;
}
void Q7BEmailWrapper::pop3finished ( int code )
{
    if( code == QxtPop3Reply::OK )
    {
        LastError = "" ;
        qDebug() << "pop3 query is successfull" ;
    }
    else
        LastError = "Pop3 error" + QString::number( code ) ;
    emit stop() ;
}

QStringList Q7BEmailWrapper::receiveNextMail(QString folderForSavingFiles )
{
    QString subject;
    QString body;

    QString s = receiveMail(_msgInfoList.last().first, subject, body, folderForSavingFiles);

    if (!s.isEmpty()) return QStringList() << s;

    QStringList l;
    l << subject;
    l << body;

    _msgInfoList.takeLast();

    return l;
}


QString Q7BEmailWrapper::receiveMail( int which ,
                                    QString & subject ,
                                    QString & body ,
                                  //  QList < QPair < QString , QByteArray > > * att ,
                                    QString folderForSavingFiles )
{
     QList < QPair < QString , QByteArray > > * att = 0;

    QEventLoop eventLoop ;
    if( ! pop3->isConnected() )
    {


        if( secured )
        {
            connect( pop3 , SIGNAL( authenticated()  ) , & eventLoop , SLOT( quit() ) ) ;
            pop3->connectToSecureHost( pop3ServerHost , pop3Port ) ;
        }
        else
        {
            connect( pop3 , SIGNAL( authenticated()  ) , & eventLoop , SLOT( quit() ) ) ;
            pop3->connectToHost( pop3ServerHost , pop3Port ) ;
        }
        connect( this , SIGNAL(stop()) ,& eventLoop , SLOT( quit() ) ) ;
        eventLoop.exec() ;
        if( LastError != "" ) return LastError ;
    }

    QxtPop3RetrReply * retrMsg = pop3->retrieveMessage( which ) ;
    connect(  retrMsg , SIGNAL(finished(int)) , SLOT(pop3finished(int))) ;
    connect( this , SIGNAL( stop() ) , & eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;
    if( LastError != "" )
        return LastError ;
    QxtMailMessage * msg = retrMsg->message() ;
    QString s = msg->rfc2822() ;
    int p =  s.indexOf( "subject: " ) ;
    if( p != - 1)
    {
        subject = s.mid( p + 9 , s.indexOf( "\r\n" , p + 9 ) - p - 9 ) ;

    }
    body = msg->body() ;
    if( att ) att->clear() ;
    for( int i = 0 ; i < msg->attachments().count() ; i ++ )
    {
        QString attName = msg->attachments().keys()[ i ] ;
        QByteArray attData = msg->attachments()[ attName ].rawData() ;
        if( att )
            att->append( QPair < QString , QByteArray > ( attName , attData ) ) ;
        if( folderForSavingFiles != "")
        {
            QDir dir( folderForSavingFiles ) ;

            if(  dir.exists() || QDir().mkpath( folderForSavingFiles ) )
            {
                if( folderForSavingFiles.right( 1 ) != "/" )
                    folderForSavingFiles.append( "/" ) ;
                QFile file( folderForSavingFiles + attName ) ;
                if( file.open( QIODevice::WriteOnly ) )
                {
                    file.write( attData ) ;
                    file.close() ;
                }
            }
        }
    }

    delete msg ;
    pop3->clearReplies() ;
    return "" ;
}

int Q7BEmailWrapper::listMailCount()
{

    QString ret = listMail( _msgInfoList ) ;

    if (!ret.isEmpty()) return -1;

  //  int sdf = msgInfoList.count();
    return _msgInfoList.count();
}

QString Q7BEmailWrapper::listMail( QList < QPair < int , int > > & msgInfoList )
{
    QEventLoop eventLoop ;
    if( ! pop3->isConnected() )
    {
        if( secured )
        {
            connect( pop3 , SIGNAL( authenticated()  ) , & eventLoop , SLOT( quit() ) ) ;
            pop3->connectToSecureHost( pop3ServerHost , pop3Port ) ;
        }
        else
        {
            connect( pop3 , SIGNAL( authenticated()  ) , & eventLoop , SLOT( quit() ) ) ;
            pop3->connectToHost( pop3ServerHost , pop3Port ) ;
        }
        connect( this , SIGNAL(stop()) , & eventLoop , SLOT( quit() ) ) ;
        eventLoop.exec() ;
        if( LastError != "" ) return LastError ;
    }
    QxtPop3ListReply * msgLst = pop3->messageList() ;
    connect(  msgLst , SIGNAL(finished(int)) , SLOT(pop3finished(int))) ;
    connect( this , SIGNAL( stop() ) , & eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;
    if( LastError != "" )
        return LastError ;

    QList<QxtPop3Reply::MessageInfo> infoList = msgLst->list() ;


    msgInfoList.clear() ;
    foreach( QxtPop3Reply::MessageInfo info , infoList )
    {
        msgInfoList.append( QPair< int , int > ( info.id , info.size) ) ;
    }
    pop3->clearReplies() ;

    return "" ;
}


