

/*
'***************************************************************************
'Copyright (C) 2010-2012 KBasic Software: www.kbasic.com. All rights reserved.
'Source codes are created by Bernd Noetscher (or company employees) sales@kbasic.com
'
'You may use this source codes under the terms of the GNU Public License (Version 3) as published
'by the Free Software Foundation.
'
'If you are interested in using the source codes for other licenses or
'commercial development, you must buy commercial licenses from KBasic Software.
'***************************************************************************
*/

#include "Q7BFtpWrapper.h"

Q7BFtpWrapper::Q7BFtpWrapper( QObject * parent ) : QObject( parent )
{
    ftp = new QFtp( this ) ;
    connect( ftp , SIGNAL( listInfo( QUrlInfo ) ) , SLOT( listInfoSlot( QUrlInfo ) ) ) ;
}

QString Q7BFtpWrapper::FtpUpload( QString sourcefilename , QString host , int port , QString user , QString password , QString toserverurl )
{
    //connecting to server
    if( host.left( 6 ) == "ftp://" )
        host.remove( 0 , 6 ) ;
    ftp->connectToHost( host , port ) ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;

    //trying to login
    ftp->login( user , password ) ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;

    //Checking for local file existing
    if( ! QFileInfo( sourcefilename ).exists() )
        return tr( "From path does not exist :(" ) ;

    if( toserverurl.left( 6 ) == "ftp://" )
        toserverurl.remove( 0 , 6 ) ;


    QStringList files , dirs  ;
    if( QFileInfo( sourcefilename ).isDir() )
    {
        //reading dirs and files lists if localFile is dir
        createFilesPathsAndDirsList( sourcefilename , files , dirs ) ;
        //Creating remote dirs
        QDir srcdir( sourcefilename ) ;
        srcdir.makeAbsolute() ;
        if( toserverurl.right( 1 ) == "/" )
            toserverurl.remove( toserverurl.length() - 1 , 1 ) ;
        foreach( QString cp , dirs )
        {
            QString rDP = srcdir.dirName() + "/" + srcdir.relativeFilePath( cp ) ;
            rDP = toserverurl + "/" + rDP ;
            ftp->mkdir( rDP ) ;
            if( waitForFtpAndCheckError() != ""  )
                if( lastError != "Creating directory failed:\nDirectory already exists" )
                    return lastError ;
        }
        //trying to upload
        foreach( QString cp , files )
        {
            QFile localFile( cp ) ;
            if( ! localFile.open( QIODevice::ReadOnly ) )
                return tr( "Can not open local file for reading :(" ) ;
            QString rDP = srcdir.dirName() + "/" + srcdir.relativeFilePath( cp ) ;
            rDP = toserverurl + "/" + rDP ;
            ftp->put( & localFile , rDP , QFtp::Binary ) ;
            if( waitForFtpAndCheckError() != "" )
                return lastError ;
        }
    }
    else
    {
        //opening local file for read
        QFile localFile( sourcefilename ) ;
        if( ! localFile.open( QIODevice::ReadOnly ) )
            return tr( "Can not open local file for reading :(" ) ;
        QString rmtPath ;
        if( toserverurl.right( 1 ) == "/" )
        {
            if( toserverurl != "/" )
            {
                ftp->mkdir( toserverurl ) ;
                waitForFtpAndCheckError() ;
            }
            rmtPath = toserverurl + QFileInfo( sourcefilename ).fileName() ;
        }
        else
        {
            QString rmtdir = QFileInfo( toserverurl ).path() ;
            if( rmtdir != "/" )
            {
                ftp->mkdir( rmtdir ) ;
                waitForFtpAndCheckError() ;
            }
            rmtPath = toserverurl ;
        }
        //trying to upload
        ftp->put( & localFile , rmtPath , QFtp::Binary ) ;
        if( waitForFtpAndCheckError() != "" )
            return lastError ;
    }

    //closing connection
    ftp->close() ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;

    return "" ;
}

QString Q7BFtpWrapper::FtpDownload( QString fromurl , QString host , int port , QString user , QString password , QString tolocalfile )
{
    //connecting to server
    if( host.left( 6 ) == "ftp://" )
        host.remove( 0 , 6 ) ;
    ftp->connectToHost( host , port ) ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;

    //trying to login
    ftp->login( user , password ) ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;
    //some correcting of path
    if( fromurl.left( 6 ) == "ftp://" )
        fromurl.remove( 0 , 6 ) ;
    if( fromurl.left( 1 ) != "/" )
        fromurl.prepend( '/' ) ;
    if( fromurl.right( 1 ) == "/" )
        fromurl.remove( fromurl.length() - 1 , 1 ) ;

    //clear files list
    dwnldFiles.clear() ;

    //listing parent dir of fromurl to detect what is it
    if( listFtpDir( QFileInfo( fromurl ).path() , false ) != "" )
        return lastError ;

    //checking if remote path is dir or file. fromurl == "/" is a special case because it doesn't have parent
    if( curLst[ QFileInfo( fromurl ).completeBaseName() ] == 1 || fromurl == "/" )
    {
        //dir case. Creating complete list of files in target dir
        if( fromurl.right( 1 ) != "/" )
            fromurl.append( "/" ) ;
        if( listFtpDir( fromurl , true ) != "" )
            return lastError ;

        //iterating through files list
        foreach( QString rmtFilePath , dwnldFiles )
        {
            //creating correct local path. fromurl == "/" is a special case
            QDir srcdir( fromurl ) ;
            QString lp ;
            if( tolocalfile.right( 1 ) == "/" )
                tolocalfile.remove( tolocalfile.length() - 1 , 1 ) ;
            if( fromurl != "/" )
                    lp = tolocalfile
                    + "/" + QFileInfo( fromurl.left( fromurl.length() - 1 ) ).fileName() + "/" +
                    srcdir.relativeFilePath( rmtFilePath ) ;
            else
                lp = tolocalfile
                        + "/" +
                        srcdir.relativeFilePath( rmtFilePath ) ;

            //creating dirs
            QDir dir ;
            dir.mkpath( QFileInfo( lp ).absolutePath() ) ;

            //opening local file for write
            QFile localFile( lp ) ;
            if( ! localFile.open( QIODevice::WriteOnly ) )
                return tr( "Can not open local file for writing :(" ) ;

            //downloading
            ftp->get( rmtFilePath , & localFile , QFtp::Binary ) ;
            if( waitForFtpAndCheckError() != "" )
                return lastError ;
        }
    }
    else
    {
        //single file case
        //creating correct local path.
        QString lp ;
        QDir dir ;

        //check if tolocalfile directory exists
        if( tolocalfile.right( 1 ) != "/" &&
                QFileInfo( tolocalfile ).isDir() && QFileInfo( tolocalfile ).exists() )
        {
            lp = tolocalfile + "/" + QFileInfo( fromurl ).fileName () ;
        }
        else if( tolocalfile.right( 1 ) == "/" )
        {
            dir.mkpath( tolocalfile ) ;
            lp = tolocalfile + QFileInfo( fromurl ).fileName () ;
        }
        else
        {
            dir.mkpath( QFileInfo( tolocalfile ).absolutePath() ) ;
            lp = tolocalfile ;
        }

        //opening local file for write
        QFile localFile( lp ) ;
        if( ! localFile.open( QIODevice::WriteOnly ) )
            return tr( "Can not open local file for writing :(" ) ;

        //downloading
        ftp->get( fromurl , & localFile , QFtp::Binary ) ;
        if( waitForFtpAndCheckError() != "" )
            return lastError ;
    }

    //closing connection
    ftp->close() ;
    if( waitForFtpAndCheckError() != "" )
        return lastError ;

    return "" ;
}

//waites until ftp finishes all operations and returns error string
QString Q7BFtpWrapper::waitForFtpAndCheckError()
{
    QEventLoop eventLoop ;
    connect( ftp , SIGNAL( done( bool ) ) , & eventLoop , SLOT( quit() ) ) ;
    eventLoop.exec() ;
    if( ftp->error() != QFtp::NoError )
        return lastError = ftp->errorString() ;
    else
        return "" ;
}

//creates lists of files and dirs inside dir p
bool Q7BFtpWrapper::createFilesPathsAndDirsList( QString p , QStringList & files , QStringList & dirs )
{
    if( ! QFileInfo( p ).isDir() )
        return false ;
    dirs.append( QFileInfo( p ).absoluteFilePath() ) ;
    QDir dir ;
    dir.cd( p ) ;
    dir.makeAbsolute() ;
    QStringList ls = dir.entryList( QDir::AllEntries | QDir::NoDotAndDotDot ) ;
    foreach( QString cp , ls )
    {
        if( QFileInfo( p + "/" + cp ).isDir() )
            createFilesPathsAndDirsList( p + "/" + cp , files , dirs ) ;
        else
            files.append( dir.absolutePath() + "/" + cp ) ;
    }
    return true ;
}

//this slot is called for each founded entry in ftp directory
void Q7BFtpWrapper::listInfoSlot ( const QUrlInfo & i )
{
    QString n = i.name() ;
    bool b = i.isDir() ;
    curLst.insert( n , b ) ;
}

//lists curPath and stores founded files in dwnldFiles
QString Q7BFtpWrapper::listFtpDir ( QString curPath , bool r )
{
    if( r )
    {
        ftp->cd( curPath ) ;
        ftp->list( curPath ) ;

        curLst.clear() ;
        if( waitForFtpAndCheckError() != "" )
            return lastError ;

        QMap< QString , bool > curLstCp = curLst ;
        for( int i = 0 ; i < curLstCp.count() ; i ++)
        {
            QString s = curLstCp.keys()[ i ] ;
            if ( s == "." || s == ".." )
                continue ; //skipping unneeded entries
            if( curLstCp.values()[ i ] ) // this is dir
                listFtpDir( curPath + s + "/" , true ) ;
            else
                dwnldFiles.append( curPath + s ) ;
        }
        return "" ;
    }
    else
    {
        ftp->cd( curPath ) ;
        ftp->list( curPath ) ;
        curLst.clear() ;
        if( waitForFtpAndCheckError() != "" )
            return lastError ;


        return "" ;
    }
}
