
/*
'***************************************************************************
'Copyright (C) 2010-2012 KBasic Software: www.kbasic.com. All rights reserved.
'Source codes are created by Bernd Noetscher (or company employees) sales@kbasic.com
'
'You may use this source codes under the terms of the GNU Public License (Version 3) as published
'by the Free Software Foundation.
'
'If you are interested in using the source codes for other licenses or
'commercial development, you must buy commercial licenses from KBasic Software.
'***************************************************************************
*/

#include "csvwrapper.h"

QList< QStringList > ReadCSV( QString filename , bool autodetect , QString _separator , bool firstlineheader )
{
    char separator = ';';
    if (!_separator.isEmpty()) separator = _separator.at(0).toAscii();

    QFile file( filename ) ;
    if( ! file.open( QIODevice::ReadOnly ) )
        return QList< QStringList > () ;
    if( autodetect )
    {
        QTextStream stream( &file ) ;
        QString str ;
        while( ( str = stream.readLine() ) != "" )
        {
            QRegExp r( "\".\"" ) ;
            int i = r.indexIn( str ) ;
            if( i != - 1 )
            {
                separator = str[ i + 1 ].toAscii() ;
                break ;
            }
        }
        file.close() ;
        file.open( QIODevice::ReadOnly ) ;
    }

    QxtCsvModel * model = new QxtCsvModel( & file  , 0 , firstlineheader , separator) ;
    QList< QStringList > ret ;
    int rCount = model->rowCount() ;
    int cCount = model->columnCount() ;
    for( int r = 0 ; r < rCount ; r ++ )
    {
        QStringList cr ;
        for( int c = 0 ; c < cCount ; c ++ )
            cr.append( model->data( model->index( r , c ) ).toString() ) ;
        ret.append( cr ) ;
    }
    delete model ;
    return ret ;
}

QList< QStringList > ReadCSVFixedsize( QString filename , int fieldwidth )
{
    QFile file( filename ) ;
    if( ! file.open( QIODevice::ReadOnly ) )
        return QList< QStringList > () ;

    QList< QStringList > ret ;
    QTextStream stream( &file ) ;
    QString str ;
    while( ( str = stream.readLine() ) != "" )
    {
        QStringList cl ;
        QString t ;
        while ( ( t = str.mid( 0 , fieldwidth ) ) != "" )
        {
            str.remove( 0 , fieldwidth ) ;
            cl.append( t.trimmed() ) ;
        }
        ret.append( cl ) ;
    }
    return ret ;
}



bool WriteCSV( QString filename , QList< QStringList > data , QString _separator , bool firstlineheader )
{
    char separator = ';';
    if (!_separator.isEmpty()) separator = _separator.at(0).toAscii();

    QFile file( filename ) ;
    if( ! file.open( QIODevice::WriteOnly ) )
        return false ;
    QxtCsvModel * model = new QxtCsvModel( & file  , 0 , firstlineheader , separator) ;
    model->insertRows( 0 , data.count() ) ;
    model->insertColumns( 0 , data[0].count() ) ;
    int r = 0 ;
    int c = 0 ;
    foreach( QStringList row , data )
    {
        c = 0 ;
        foreach( QString field , row )
        {
            model->setData( model->index( r , c ) , field ) ;
            c ++ ;
        }
        r ++ ;
    }
    model->toCSV( & file , firstlineheader , separator ) ;
    delete model ;
    return true ;
}

bool WriteCSVFixedsize( QString filename , QList< QStringList > data , int fieldwidth )
{
    QFile file( filename ) ;
    if( ! file.open( QIODevice::WriteOnly ) )
        return false ;
    QTextStream stream( &file ) ;
    stream.setFieldWidth( fieldwidth ) ;
    foreach( QStringList cl , data )
    {
        foreach( QString t , cl )
            stream << t ;
        stream << endl ;
    }
    return true ;
}

QMap< QString , QString > InfoCSV( QString filename )
{
    QMap < QString , QString > ret ;
    QFile file( filename ) ;
    char separator = 0 ;
    if( ! file.open( QIODevice::ReadOnly ) )
        return ret ;

    QTextStream stream( &file ) ;
    QString str ;
    int lineCount = 0 ;
    int maxLength = 0 ;
    while( ( str = stream.readLine() ) != "" )
    {
        if( maxLength < str.length() )
            maxLength = str.length() ;
        lineCount ++ ;
        QRegExp r( "\".\"" ) ;
        int i = r.indexIn( str ) ;
        if( i != - 1 )
        {
            separator = str[ i + 1 ].toAscii() ;
        }
    }
    ret.insert( "separator" , QChar( separator ) ) ;
    ret.insert( "line count" , QString::number( lineCount ) ) ;
    ret.insert( "max line length" , QString::number( lineCount ) ) ;
    return  ret ;
}
