#include "basicmethodtranslator.h"

BasicMethodTranslator::BasicMethodTranslator( QMetaMethod  & method )
{
    fName = QString ( method.signature() ).mid( 0 , QString(method.signature()).indexOf( "(" ) ) ;
    fType = method.methodType() ;
    fRetType = translateToBasicType ( method.typeName() ) ;
    int curArgNumber = 1 ;

    foreach ( QString name , method.parameterNames() )
    {
        if ( name == "" )
        {
            if ( method.parameterNames().count() > 1 )
                name = QString( "arg%1" ).arg ( curArgNumber ++ ) ;
            else
                name = "arg" ;
        }
        fArgNames.append ( name ) ;
    }
    foreach ( QString name , method.parameterTypes() )
    {
        name = translateToBasicType ( name ) ;
        fArgTypes.append ( name ) ;
    }
}

QString BasicMethodTranslator::basicLangVariant ()
{
    QString ret = fName ;
    ret += "(" ;
    if ( ! fArgNames.isEmpty() )
    {
        for ( int i = 0 ; i < fArgNames.count () ; i ++ )
        {
            ret += fArgNames [ i ] ;
            ret += " As " ;
            ret += fArgTypes [ i ] ;
            if ( i != fArgNames.count () - 1 )
                ret += ", " ;
        }
    }
    ret += ")" ;
    if ( fRetType != "" )
        ret += " As " + fRetType ;
    return ret ;
}

bool BasicMethodTranslator::isSignal ()
{
    return fType == QMetaMethod::Signal ;
}

BasicMethodTranslator::BasicMethodTranslator( const BasicMethodTranslator & a)
{
    fArgNames = a.fArgNames ;
    fArgTypes = a.fArgTypes ;
    fName = a.fName ;
    fType = a.fType ;
    fRetType = a.fRetType ;
}

QMetaMethod::MethodType BasicMethodTranslator::type ()
{
    return fType ;
}

QStringList BasicMethodTranslator::argNames ()
{
    return fArgNames ;
}

QStringList BasicMethodTranslator::argTypes ()
{
    return fArgTypes ;
}

QString BasicMethodTranslator::name()
{
    return fName ;
}

QString BasicMethodTranslator::retType()
{
    return fRetType ;
}

QList < BasicMethodTranslator > BasicMethodTranslator::fromQObject(QObject * w)
{
    QList < BasicMethodTranslator > ret ;
    if ( w )
    {
        for ( int i = 0 ; i < w->metaObject()->methodCount() ; i ++ )
        {
            QMetaMethod m = w->metaObject()->method ( i ) ;
            ret.append ( BasicMethodTranslator ( m ) ) ;
        }
    }
    return ret ;
}

QString BasicMethodTranslator::translateToBasicType ( QString type )
{
    static QHash < QString , QString > basicTypesTranslations ;
    if (basicTypesTranslations.isEmpty())
    {
        basicTypesTranslations [ "float" ] = "Float" ;
        basicTypesTranslations [ "double" ] = "Float" ;
        basicTypesTranslations [ "double" ] = "Float" ;
        basicTypesTranslations [ "int" ] = "Integer" ;
        basicTypesTranslations [ "bool" ] = "Boolean" ;
        basicTypesTranslations [ "QString" ] = "String" ;
    }
    type = type.simplified() ;
    type.remove ( QChar ( '*' ) ) ;
    if ( basicTypesTranslations.contains( type ) )
        type = basicTypesTranslations.value( type ) ;
    return type ;
}
