#include "propertyeditor.h"
#include "selectionsquares.h"
#include "widgetmanager.h"
#include <QHBoxLayout>
#include <QHeaderView>
#include <QMetaProperty>
#include <QFontDialog>
#include <QDebug>
#include "constants.h"
#include "props_functions.h"
#include "qobpropmodel.h"
PropertyEditor::PropertyEditor  ( QWidget *parent) :
    QWidget  (parent) , WM  ( 0 )
{
    QHBoxLayout *l = new QHBoxLayout  ( this ); // BN
    l->setSpacing(0); // BN
    l->setContentsMargins(0, 0, 0, 0); // BN

    setLayout  ( l ) ;
    //view = new QTableView  ( this ) ;
    view = new QTreeView( this ) ;
    deleg = new PropertyDelegate ;

    view->setItemDelegate  ( deleg ) ;
    view->setSizePolicy  ( QSizePolicy::Minimum , QSizePolicy::Expanding ) ;
    //view->horizontalHeader  ()->setStretchLastSection  ( true ) ;
    view->header()->setResizeMode( QHeaderView::ResizeToContents );
    view->show  () ;
    layout  ()->addWidget ( view ) ;
    //selectionIsChanging = false ;
}

void PropertyEditor::selectionChanged (  )
{
    if ( ! WM )
        return ;
    //selectionIsChanging = true ;

    if ( ! WM->getCurrentSelList ().isEmpty () )
    {
        QWidget * w = WM->getCurrentSelList () [ 0 ] ;

        if ( ! w )
            return ;
        if ( view->model () )
        {
            QModelIndex i = view->model ()->index(0,0);
            for ( int i = 0 ; i < view->model ()->rowCount() ; i ++ )
                expandState [ view->model()->data(view->model ()->index(i,0) ).toString() ]
                        = view->isExpanded( view->model ()->index(i,0) ) ;

            view->model ()->deleteLater () ;
        }
        //ObjectPropertiesModel * model = new ObjectPropertiesModel ( w , this ) ;
        QObPropModel * model = new QObPropModel( w , this ) ;
        view->setModel ( model ) ;
        connect( deleg , SIGNAL(fontEdited(const QModelIndex&,QFont)) , model , SLOT(fontEdited(const QModelIndex&,QFont)) ) ;
        connect ( model , SIGNAL (propsEdited (QVariantMap)) , this , SIGNAL (propsEdited (QVariantMap)) ) ;
        for ( int i = 0 ; i < view->model ()->rowCount() ; i ++ )
            if ( expandState.contains( view->model()->data( view->model ()->index(i,0) ).toString() ) )
                view->setExpanded( view->model ()->index(i,0)
                                   , expandState [ view->model()->data(view->model ()->index(i,0) ).toString() ] )  ;
    }
    else
    {
        if ( view->model () )
            view->model ()->deleteLater () ;
    }
    //selectionIsChanging = false ;
}

void PropertyEditor::wmChanged ( WidgetManager * w )
{
    WM = w ;
    if ( WM )
        selectionChanged (  ) ;
}


QWidget *PropertyDelegate::createEditor (QWidget *parent,
                                    const QStyleOptionViewItem &option,
                                    const QModelIndex &index) const

{
    if  ( index.data (Qt::EditRole).type () == QVariant::Rect )
        return new GeometryEditor (parent);
    else if ( index.data (Qt::EditRole).type () == QVariant::Size )
        return new SizeEditor (parent) ;
    else if ( index.data (Qt::EditRole).type () == QVariant::Point )
        return new PointEditor (parent) ;
    else if ( index.data (Qt::EditRole).type () == QVariant::Font )
    {
        FontEditor * e =  new FontEditor ( qVariantValue< QFont > ( index.data (Qt::EditRole) )
                                           , index
                                           , parent ) ;
        connect ( e  , SIGNAL (finished (QWidget*)) , this ,
                 SLOT (fin (QWidget*))) ;
        return e ;
    }
    else
        return QStyledItemDelegate::createEditor (parent, option, index);
}

void PropertyDelegate::setEditorData (QWidget *editor,
                                 const QModelIndex &index) const
{
    if  ( index.data (Qt::EditRole).type () == QVariant::Rect )
    {
        QRect rect = qVariantValue<QRect> ( index.data (Qt::EditRole) ) ;
        GeometryEditor * rectEditor = qobject_cast<GeometryEditor *> (editor);
        rectEditor->setRect (rect);
    }
    else if  ( index.data (Qt::EditRole).type () == QVariant::Size )
    {
        QSize size = qVariantValue<QSize> ( index.data (Qt::EditRole) ) ;
        SizeEditor * sizeEditor = qobject_cast<SizeEditor *> (editor);
        sizeEditor->setSize (size);
    }
    else if  ( index.data (Qt::EditRole).type () == QVariant::Point )
    {
        QPoint point = qVariantValue<QPoint> ( index.data (Qt::EditRole) ) ;
        PointEditor * pointEditor = qobject_cast<PointEditor *> (editor);
        pointEditor->setPoint (point);
    }
    else if  ( index.data (Qt::EditRole).type () == QVariant::Font )
    {
        FontEditor * fontEditor = qobject_cast<FontEditor *> (editor);
        fontEditor->setFont ( index.data (Qt::EditRole ) ) ;
    }
    else
    {
        QStyledItemDelegate::setEditorData (editor, index);
    }
}

void PropertyDelegate::setModelData (QWidget *editor, QAbstractItemModel *model,
                                const QModelIndex &index) const
{
    if  (index.data (Qt::EditRole).type () == QVariant::Rect)
    {
        GeometryEditor *rectEditor = qobject_cast<GeometryEditor *> (editor);
        model->setData (index, qVariantFromValue (rectEditor->getRect ()));
    }
    else if  (index.data (Qt::EditRole).type () == QVariant::Size)
    {
        SizeEditor *sizeEditor = qobject_cast<SizeEditor *> (editor);
        model->setData (index, qVariantFromValue (sizeEditor->getSize ()));
    }
    else if  (index.data (Qt::EditRole).type () == QVariant::Point)
    {
        PointEditor *pointEditor = qobject_cast<PointEditor *> (editor);
        model->setData (index, qVariantFromValue (pointEditor->getPoint ()));
    }
    else if  (index.data (Qt::EditRole).type () == QVariant::Font)
    {

    }
    else
    {
        QStyledItemDelegate::setModelData (editor, model, index);
    }
}

void PropertyDelegate::fin( QWidget * w )
{
    FontEditor *fontEditor = qobject_cast<FontEditor *> (w);
    lastEditedFont = fontEditor->getFont () ;
    const QModelIndex ind = fontEditor->getIndex() ;
    QFontDialog fd( lastEditedFont ) ;
    fd.setOption  ( QFontDialog::DontUseNativeDialog ) ;

    fd.exec  () ;
    //font = fd.selectedFont () ;

    emit fontEdited( ind , fd.selectedFont () ) ;
}


GeometryEditor::GeometryEditor (QWidget *parent) : QWidget (parent)
{
    setLayout ( new QHBoxLayout ( this ) ) ;
    layout ()->setMargin ( 0 ) ;
    widthEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( widthEdit ) ;
    widthEdit->setPrefix ( "w: " ) ;
    widthEdit->setMaximum ( 5000 ) ;
    widthEdit->show () ;

    heightEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( heightEdit ) ;
    heightEdit->setPrefix ( "h: " ) ;
    heightEdit->setMaximum ( 5000 ) ;
    heightEdit->show () ;

    leftEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( leftEdit ) ;
    leftEdit->setPrefix ( "l: " ) ;
    leftEdit->setMaximum ( 5000 ) ;
    leftEdit->show () ;

    topEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( topEdit ) ;
    topEdit->setPrefix ( "t: " ) ;
    topEdit->setMaximum ( 5000 ) ;
    topEdit->show () ;
}

void GeometryEditor::setRect ( QRect r )
{
    leftEdit->setValue ( r.left () );
    topEdit->setValue ( r.top () );
    widthEdit->setValue ( r.width () );
    heightEdit->setValue ( r.height () );
}

QRect GeometryEditor::getRect ()
{
    return QRect ( leftEdit->value () , topEdit->value () , widthEdit->value () , heightEdit->value () ) ;
}

void GeometryEditor::paintEvent (QPaintEvent *)
{
    QPainter painter ( this ) ;
    painter.setPen ( QPen ( QBrush ( Qt::white ) , 1 ) ) ;
    painter.setBrush ( QBrush ( Qt::white) ) ;
    painter.drawRect (  0 , 0 , width () , height () ) ;
}

SizeEditor::SizeEditor (QWidget *parent) : QWidget (parent)
{
    setLayout ( new QHBoxLayout ( this ) ) ;
    layout ()->setMargin ( 0 ) ;
    widthEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( widthEdit ) ;
    widthEdit->setPrefix ( "w: " ) ;
    widthEdit->setMaximum ( 5000 ) ;
    widthEdit->show () ;

    heightEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( heightEdit ) ;
    heightEdit->setPrefix ( "h: " ) ;
    heightEdit->setMaximum ( 5000 ) ;
    heightEdit->show () ;
}

void SizeEditor::setSize ( QSize s )
{
    widthEdit->setValue ( s.width () );
    heightEdit->setValue ( s.height () );
}

QSize SizeEditor::getSize ()
{
    return QSize ( widthEdit->value () , heightEdit->value () ) ;
}

void SizeEditor::paintEvent (QPaintEvent *)
{
    QPainter painter ( this ) ;
    painter.setPen ( QPen ( QBrush ( Qt::white ) , 1 ) ) ;
    painter.setBrush ( QBrush ( Qt::white) ) ;
    painter.drawRect (  0 , 0 , width () , height () ) ;
}

PointEditor::PointEditor (QWidget *parent) : QWidget (parent)
{
    setLayout ( new QHBoxLayout ( this ) ) ;
    layout ()->setMargin ( 0 ) ;
    xEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( xEdit ) ;
    xEdit->setPrefix ( "x: " ) ;
    xEdit->setMaximum ( 5000 ) ;
    xEdit->show () ;

    yEdit = new QSpinBox ( this ) ;
    layout ()->addWidget ( yEdit ) ;
    yEdit->setPrefix ( "y: " ) ;
    yEdit->setMaximum ( 5000 ) ;
    yEdit->show () ;
}

void PointEditor::setPoint ( QPoint p )
{
    xEdit->setValue ( p.x () );
    yEdit->setValue ( p.y () );
}

QPoint PointEditor::getPoint ()
{
    return QPoint ( xEdit->value () , yEdit->value () ) ;
}

void PointEditor::paintEvent (QPaintEvent *)
{
    QPainter painter ( this ) ;
    painter.setPen ( QPen ( QBrush ( Qt::white ) , 1 ) ) ;
    painter.setBrush ( QBrush ( Qt::white) ) ;
    painter.drawRect (  0 , 0 , width () , height () ) ;
}

FontEditor::FontEditor ( QFont f , const QModelIndex &i , QWidget *parent) : QWidget (parent)
{
    index = i ;
    setLayout ( new QHBoxLayout ( this ) ) ;
    layout ()->setMargin ( 0 ) ;
    l = new QLabel ( this ) ;
    layout ()->addWidget ( l ) ;
    l->show () ;

    b = new QPushButton ( "..." , this ) ;
    b->setMaximumWidth ( b->height () ) ;
    layout ()->addWidget ( b ) ;
    timer.setSingleShot ( true ) ;
    timer.setInterval ( 50 ) ;
    connect ( & timer , SIGNAL ( timeout() ) , SLOT ( buttonPress () ) , Qt::QueuedConnection ) ;
    timer.start () ;
    b->show () ;

    font = f ;
    //buttonPress() ;
    //QTimer::singleShot( 0 , this , SLOT(buttonPress()) , Qt::QueuedConnection ) ;
}

void FontEditor::setFont ( QVariant f )
{
    l->setText ( f.toString () ) ;
    font = qVariantValue< QFont > ( f ) ;
}

QFont FontEditor::getFont ()
{

    return font ;
}

void FontEditor::paintEvent ( QPaintEvent * )
{
    QPainter painter ( this ) ;
    painter.setPen ( QPen ( QBrush ( Qt::white ) , 1 ) ) ;
    painter.setBrush ( QBrush ( Qt::white) ) ;
    painter.drawRect (  0 , 0 , width () , height () ) ;
}

void FontEditor::buttonPress ()
{
    emit finished ( this ) ;
}
const QModelIndex & FontEditor::getIndex()
{
    return index ;
}





