#include "props_functions.h"
#include "constants.h"
#include <QVariantList>
#include <QStringList>
#include <QMetaProperty>
void PF::setPropReadOnly ( QObject * ob , QString name )
{
    QVariantList roList = ob->property ( READ_ONLY_LIST_PROP_NAME ).toList () ;
    roList << name ;
    ob->setProperty ( READ_ONLY_LIST_PROP_NAME , roList ) ;
}

bool PF::checkPropIsReadOnly ( QObject * ob , QString name )
{
    if( ob->property ( READ_ONLY_LIST_PROP_NAME ) != QVariant() )
    {
        QVariantList roList = ob->property ( READ_ONLY_LIST_PROP_NAME ).toList () ;
        return roList.contains ( name ) ;
    }
    return false ;
}
bool PF::checkPropIsReadOnly ( QObject * ob , int index )
{
    return checkPropIsReadOnly( ob , metaPropertyName( ob , index ) ) ;
}

bool  PF::checkPropIsInternal ( QString name )
{
    static QStringList internalProps ;
    if( internalProps.isEmpty() )
        internalProps << READ_ONLY_LIST_PROP_NAME
                      << DELETED_PROP_NAME
                      << TABINDEX_PROP_NAME
                      << INTERNAL_PROPS_LIST_NAME
                      << METAPROPERTY_REPLACEMENTS
                      << "WidgetClassName"
                      << Z_ORDER
                      << LAYOUT_KIND_PROP_NAME ;

    return internalProps.contains ( name ) || name.mid ( 0 , 2 ) == "_q" ;
}

int PF::visibleDynPropsCount ( QObject * ob )
{
    QList < QByteArray > list = ob->dynamicPropertyNames() ;
    int count = 0 ;
    foreach ( QString n , list )
        if( ! checkPropIsInternal( n ) )
            ++ count ;
    return count ;
}

QString PF::getVisibleDynPropName ( int index , QObject * ob )
{
    QList < QByteArray > list = ob->dynamicPropertyNames() ;
    int count = 0 ;
    foreach ( QString n , list )
        if( ! checkPropIsInternal( n ) )
        {
            if( count == index )
                return n ;
            ++ count ;
        }
    return "" ;
}

QVariant PF::readMetaProperty ( const QObject * w , QString name )
{
    if ( ! w->property( METAPROPERTY_REPLACEMENTS ).toMap().contains( name ) )
        return w->metaObject()->property(
                w->metaObject()->indexOfProperty( name.toAscii().constData() ) ).read( w ) ;
    else
        return w->property ( METAPROPERTY_REPLACEMENTS ).toMap()[
                                    name ] ;

}

QVariant PF::readMetaProperty ( QObject * w , int index )
{
    return readMetaProperty( w , metaPropertyName( w , index ) ) ;
}

bool PF::hasReplacement (  QObject * w , QString name )
{
    return w->property( METAPROPERTY_REPLACEMENTS ).toMap().contains( name ) ;
}

bool PF::writeMetaPropertyD( QObject * w , QString name , QVariant value )
{
    int i = w->metaObject()->indexOfProperty( name.toAscii().constData() ) ;
    if ( i != -1 )
    {
        w->metaObject()->property( i ).write( w , value ) ;
        return true ;
    }
    else
        return false ;
}

bool PF::writeMetaProperty( QObject * w , QString name , QVariant value )
{
    if ( ! w->property( METAPROPERTY_REPLACEMENTS ).toMap().contains( name ) )
    {
        int i = w->metaObject()->indexOfProperty( name.toAscii().constData() ) ;
        if ( i != -1 )
        {
            w->metaObject()->property( i ).write( w , value ) ;
            return true ;
        }
        else
            return false ;
    }
    else
    {
        QVariantMap mpr = w->property( METAPROPERTY_REPLACEMENTS ).toMap() ;
        mpr[ name ] = value ;
        w->setProperty( METAPROPERTY_REPLACEMENTS , mpr ) ;
        return true ;
    }
}

bool PF::writeMetaProperty( QObject * w , int index , QVariant value )
{
    return writeMetaProperty( w , metaPropertyName( w , index ) , value ) ;
}

int PF::metaPropertyCount( QObject * w )
{
    return w->metaObject()->propertyCount() ;
}

int PF::metaPropertyIndex( QObject * w , QString name )
{
    return w->metaObject()->indexOfProperty( name.toAscii().constData() ) ;
}

QString PF::metaPropertyName( QObject * w , int index )
{
    return w->metaObject()->property( index ).name() ;
}

QVariant::Type PF::metaPropertyType ( QObject * w , QString name )
{
    return w->metaObject ()->property ( w->metaObject()->indexOfProperty(
                                     name.toAscii().constData() ) ).type () ;
}

QVariant::Type PF::metaPropertyType ( QObject * w , int index )
{
    return metaPropertyType ( w , metaPropertyName( w , index ) ) ;
}

bool PF::writeProperty ( QObject * w , QString name , QVariant value )
{
    if( checkPropIsInternal( name ) )
        return false ;
    if ( writeMetaProperty( w , name , value ) )
        return true ;
    else
    {
        w->setProperty( name.toAscii().constData() , value ) ;
        return true ;
    }

}

QVariant PF::readProperty ( const QObject * w , QString name )
{
    if( checkPropIsInternal( name ) )
        return QVariant () ;
    QVariant v = readMetaProperty( w , name ) ;
    if ( v != QVariant () )
        return v ;
    else if ( w->dynamicPropertyNames().contains( name.toAscii() ) )
        return w->property( name.toAscii().constData() ) ;
    else return QVariant () ;
}
bool PF::checkPropertyWritable ( QObject * w , QString name )
{
    if( checkPropIsReadOnly( w , name ) )
        return false ;
    if ( w->metaObject()->indexOfProperty( name.toAscii().constData() ) != - 1 )
        return w->metaObject ()->property ( w->metaObject()->indexOfProperty(
                                         name.toAscii().constData() ) ).isWritable() ;
    else if( w->dynamicPropertyNames().contains( name.toAscii() ) )
        return true ;
    else return false ;
}

bool PF::checkPropertyWritable ( QObject * w , int index )
{
    return checkPropertyWritable ( w , metaPropertyName( w , index ) ) ;
}
bool PF::checkMetaPropertyWritable ( QObject * w , QString name )
{
    if( checkPropIsReadOnly( w , name ) )
        return false ;
    return w->metaObject ()->property ( w->metaObject()->indexOfProperty(
                                     name.toAscii().constData() ) ).isWritable() ;
}

bool PF::checkMetaPropertyWritable ( QObject * w , int index )
{
    return checkMetaPropertyWritable ( w , metaPropertyName( w , index ) ) ;
}

QList< QPair < QString , QStringList > > PF::groupedProperties( const QObject * w  )
{
    QMap < QString , QStringList > all ;
    QMap < QString , QString > t ;

    const QMetaObject * mo = w->metaObject() ;
    while ( mo )
    {
        QString cn = mo->className() ;
        for( int i = 0 ; i < mo->propertyCount() ; i ++ )
            t[ mo->property( i ).name() ] = cn ;
        mo = mo->superClass() ;
    }
    QMap<QString, QString>::iterator i = t.begin() ;
    while ( i != t.end() )
    {
        all[ i.value() ].append( i.key() ) ;
        i ++ ;
    }
    QList < QByteArray > list = w->dynamicPropertyNames() ;
    foreach ( QString n , list )
        if( ! checkPropIsInternal( n ) )
            all[ "Dynamic" ].append( n ) ;
    QPair < QString , QStringList > pair ;
    QList< QPair < QString , QStringList > > ret ;
    mo = w->metaObject() ;
    while ( mo )
    {
        QString cn = mo->className() ;
        pair.first = cn ;
        pair.second = all [ cn ] ;
        if ( ! all [ cn ].isEmpty() )
            ret.prepend ( pair ) ;
        mo = mo->superClass() ;
    }
    pair.first = "Dynamic" ;
    pair.second = all [ "Dynamic" ] ;
    ret.append( pair ) ;
    return ret ;
}
