#include "qobpropmodel.h"
#include "props_functions.h"
#include <QStringList>
#include <QRect>
#include <QSize>
#include <QPoint>
#include <QDebug>
#include <QPair>
#include <QColor>
#include <QBrush>
OPTreeItem::OPTreeItem ( OPTreeItem * parent  )
{
    parentItem = parent ;
    r = PROP ;
    d << 0 << 0 ;
}
OPTreeItem::~OPTreeItem()
{
    qDeleteAll(childItems);
}

OPTreeItem::Role OPTreeItem::role()
{
    return r ;
}

void OPTreeItem::setRole ( OPTreeItem::Role role )
{
    r = role ;
}

OPTreeItem *OPTreeItem::child(int number)
{
    return childItems.value(number);
}

int OPTreeItem::childCount() const
{
    return childItems.count();
}

int OPTreeItem::columnCount() const
{
    return 1 ;
}

QVariant OPTreeItem::data(int column) const
{
    if( column < d.count() && column == 0 )
        return d.value( column ) ;
    if ( column < d.count() && column == 1 && r != GROUP )
        return d.value( column ) ;
    return QVariant() ;
}

bool OPTreeItem::insertChildren(int position, int count )
{
    if (position < 0 || position > childItems.size() )
        return false;

    for ( int row = 0 ; row < count ; ++ row)
    {
        OPTreeItem *item = new OPTreeItem( this ) ;
        childItems.insert(position, item);
    }

    return true;
}
//bool OPTreeItem::insertColumns(int position, int columns);
OPTreeItem *OPTreeItem::parent()
{
    return parentItem ;
}

//bool OPTreeItem::removeChildren(int position, int count);
//bool OPTreeItem::removeColumns(int position, int columns);
int OPTreeItem::childNumber() const
{
    if (parentItem)
        return parentItem->childItems.indexOf(const_cast<OPTreeItem*>(this));
    return 0 ;
}

bool OPTreeItem::setData( int column , const QVariant &value )
{
    if (column < 0 || column >= d.size())
        return false;

    d[column] = value;
    return true;
}

QObPropModel::QObPropModel( QObject * target , QObject *parent) :
    QAbstractItemModel(parent)
{
    rootItem = new OPTreeItem ;
    trg = target ;
    setupModelData( target , rootItem);

}

QObPropModel::~QObPropModel()
{
    delete rootItem ;
}

void QObPropModel::setupModelData(const QObject * target , OPTreeItem * parent )
{
    QList< QPair < QString , QStringList > > props = PF::groupedProperties( target ) ;
    parent->insertChildren( 0 , props.count() ) ;
    int gI = 0 ;
    QPair < QString , QStringList >  gr ;
    foreach( gr , props )
    {
        QString grName = gr.first ;
        QStringList curGroupProps = gr.second ;

        OPTreeItem * grNode = parent->child( gI ++ ) ;
        grNode->setData( 0 , grName ) ;
        grNode->setRole( OPTreeItem::GROUP ) ;
        grNode->insertChildren( 0 , curGroupProps.count() ) ;
        int pI = 0 ;
        foreach( QString prName , curGroupProps )
        {
            OPTreeItem * prNode = grNode->child( pI ++ ) ;
            prNode->setData( 0 , prName ) ;
            QVariant v = PF::readProperty( target , prName ) ;
            prNode->setData( 1 , v ) ;
            prNode->setRole( OPTreeItem::PROP ) ;
            switch( v.type() )
            {
                break ; case QVariant::Rect :
                    prNode->insertChildren( 0 , 4 ) ;
                    prNode->child( 0 )->setData( 0 , "x" ) ;
                    prNode->child( 0 )->setData( 1 , v.toRect().topLeft().x() ) ;
                    prNode->child( 0 )->setRole( OPTreeItem::SUBPROP ) ;
                    prNode->child( 1 )->setData( 0 , "y" ) ;
                    prNode->child( 1 )->setData( 1 , v.toRect().topLeft().y() ) ;
                    prNode->child( 1 )->setRole( OPTreeItem::SUBPROP ) ;
                    prNode->child( 2 )->setData( 0 , "width" ) ;
                    prNode->child( 2 )->setData( 1 , v.toRect().width() ) ;
                    prNode->child( 2 )->setRole( OPTreeItem::SUBPROP ) ;
                    prNode->child( 3 )->setData( 0 , "height" ) ;
                    prNode->child( 3 )->setData( 1 , v.toRect().height() ) ;
                    prNode->child( 3 )->setRole( OPTreeItem::SUBPROP ) ;
                break ; case QVariant::Size :
                    prNode->insertChildren( 0 , 2 ) ;
                    prNode->child( 0 )->setData( 0 , "width" ) ;
                    prNode->child( 0 )->setData( 1 , v.toSize().width() ) ;
                    prNode->child( 0 )->setRole( OPTreeItem::SUBPROP ) ;
                    prNode->child( 1 )->setData( 0 , "height" ) ;
                    prNode->child( 1 )->setData( 1 , v.toSize().height() ) ;
                    prNode->child( 1 )->setRole( OPTreeItem::SUBPROP ) ;

                break ; case QVariant::Point :
                    prNode->insertChildren( 0 , 2 ) ;
                    prNode->child( 0 )->setData( 0 , "x" ) ;
                    prNode->child( 0 )->setData( 1 , v.toPoint().x() ) ;
                    prNode->child( 0 )->setRole( OPTreeItem::SUBPROP ) ;
                    prNode->child( 1 )->setData( 0 , "y" ) ;
                    prNode->child( 1 )->setData( 1 , v.toPoint().y() ) ;
                    prNode->child( 1 )->setRole( OPTreeItem::SUBPROP ) ;

                break ; case QVariant::Font :
                ;
                break ; default :
                ;
            }
        }
    }
}

OPTreeItem *QObPropModel::getItem(const QModelIndex &index) const
{
    if ( index.isValid() )
    {
        OPTreeItem * item = static_cast<OPTreeItem*>( index.internalPointer() ) ;
        if ( item ) return item ;
    }
    return rootItem ;
}

QColor QObPropModel::createRowColor ( int row , int maxRow ) const
{
    double k = 200.0 / double ( maxRow ) * double ( row ) ;
    return QColor( 240 - k , 240 - k , 240 - k  ) ;
}


QVariant QObPropModel::data(const QModelIndex &index, int role) const
{
    if ( ! index.isValid() )
        return QVariant () ;

    OPTreeItem * item = getItem(index);
    if ( role == Qt::BackgroundRole )
    {
        int totalRows = rowCount() ;
        int curRow = 0 ;
        QModelIndex ci = index ;
        while ( ci != QModelIndex() )
        {
            curRow = ci.row() ;
            ci = ci.parent() ;

        }
        QColor color = createRowColor(  curRow , totalRows ) ;
        QBrush brush ( color ) ;
        return brush ;
    }
    if  ( index.column () == 1 )
    {
        if ( item->role() == OPTreeItem::PROP )
        {
            if ( role == Qt::DisplayRole && item->data( 1 ).type() != QVariant::Bool )
            {
                QVariant v = item->data( 1 ) ;

                switch ( v.type () )
                {
                    case QVariant::Rect :
                    {
                        QRect r = v.toRect () ;
                        return QString ( "%1x%2+%3+%4" ).arg ( QString::number ( r.width () ) )
                                .arg ( QString::number ( r.height () ) )
                                .arg ( QString::number ( r.left () ) )
                                .arg ( QString::number ( r.top () ) ) ;
                        break ;
                    }
                    case QVariant::Size :
                    {
                        QSize s = v.toSize () ;
                        return QString ( "%1x%2" ).arg ( QString::number ( s.width () ) )
                                .arg ( QString::number ( s.height () ) ) ;
                        break ;
                    }
                    case QVariant::Point :
                    {
                        QPoint p = v.toPoint () ;
                        return QString ( " (%1,%2)" ).arg ( QString::number ( p.x () ) )
                                .arg ( QString::number ( p.y () ) ) ;
                        break ;
                    }
                    default :
                        return v ;
                }
            }
            else if (  ( role == Qt::CheckStateRole  ) && item->data( 1 ).type()  == QVariant::Bool )
            {
                bool v = item->data( 1 ).toBool() ;
                if ( v )
                    return 2 ;
                else
                    return 0 ;
            }
            else if ( role == Qt::EditRole )
                return item->data( 1 ) ;
            else if ( role == Qt::TextAlignmentRole )
                return Qt::AlignLeft ;
            else
                return QVariant ();
        }
        else if( item->role() == OPTreeItem::SUBPROP )
        {
            if ( ( role == Qt::DisplayRole || role == Qt::EditRole ) && item->data( 1 ).type() != QVariant::Bool  )
                return item->data( 1 ) ;
            else if (  role == Qt::CheckStateRole  )
                return QVariant() ;
            else if ( role == Qt::TextAlignmentRole )
                return Qt::AlignLeft ;

            else
                return QVariant() ;
        }
    }
    else if ( index.column () == 0 && role == Qt::DisplayRole )
        return item->data( 0 ) ;
    else if ( index.column () == 0 && role == Qt::TextAlignmentRole && item->role() != OPTreeItem::GROUP )
        return Qt::AlignLeft ;
    else if ( index.column () == 0 && role == Qt::TextAlignmentRole && item->role() == OPTreeItem::GROUP )
        return Qt::AlignLeft ;
    else
        return QVariant ();


    return item->data ( index.column() ) ;
}

QVariant QObPropModel::headerData(int section, Qt::Orientation orientation,
                                  int role ) const
{
    if ( orientation == Qt::Horizontal && role == Qt::DisplayRole )
    {
        switch( section )
        {
            case 0:
                return "Name" ;
            case 1:
                return "Value" ;
        }
    }
    return QVariant();
}

QModelIndex QObPropModel::index(int row, int column,
                                const QModelIndex &parent) const
{
    if ( parent.isValid() && parent.column() != 0 )
        return QModelIndex();
    OPTreeItem * parentItem = getItem( parent ) ;

    OPTreeItem * childItem = parentItem->child ( row ) ;
    if ( childItem )
        return createIndex ( row , column , childItem ) ;
    else
        return QModelIndex() ;
}
QObject * QObPropModel::target()
{
    return trg ;
}

QModelIndex QObPropModel::parent( const QModelIndex & index ) const
{
    if ( ! index.isValid() )
        return QModelIndex() ;

    OPTreeItem * childItem = getItem( index ) ;
    OPTreeItem * parentItem = childItem->parent() ;

    if ( parentItem == rootItem )
        return QModelIndex() ;

    return createIndex( parentItem->childNumber() , 0 , parentItem ) ;
}

int QObPropModel::rowCount( const QModelIndex &parent ) const
{
    OPTreeItem * parentItem = getItem ( parent ) ;

    return parentItem->childCount() ;
}

int QObPropModel::columnCount(const QModelIndex & ) const
{
    return 2 ;
}

Qt::ItemFlags QObPropModel::flags(const QModelIndex &index) const
{
    //rewrite
    if (!index.isValid())
        return 0;
    OPTreeItem * item = getItem ( index ) ;
    if( item->role() == OPTreeItem::GROUP )
        return Qt::ItemIsEnabled ;
    else if( index.column() == 1 )
    {
        if ( item->role() == OPTreeItem::PROP )
        {
            if ( PF::checkPropertyWritable( trg , item->data( 0 ).toString() ) )
            {

                if ( item->data( 1 ).type() != QVariant::Bool )
                    return QAbstractItemModel::flags (index) | Qt::ItemIsEditable | Qt::ItemIsSelectable;
                else
                    return  QAbstractItemModel::flags (index ) | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
            }
            else
                return QAbstractItemModel::flags (index) &  ( ~ Qt::ItemIsEnabled ) ;
        }
        else if ( item->role() == OPTreeItem::SUBPROP )
        {
            if ( PF::checkPropertyWritable( trg , item->parent()->data( 0 ).toString() ) )
            {

                if ( item->data( 1 ).type() != QVariant::Bool )
                    return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable;
                else
                    return  QAbstractItemModel::flags (index ) | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
            }
            else
                return QAbstractItemModel::flags (index) &  ( ~ Qt::ItemIsEnabled ) ;
        }
    }
    else
        return Qt::ItemIsEnabled ;
    return Qt::ItemIsEnabled ;
}

bool QObPropModel::setData(const QModelIndex &index, const QVariant &value,
                           int role )
{
    if  ( ! index.isValid () || index.column() != 1 )
        return false ;

    OPTreeItem * item = getItem ( index ) ;
    if ( item->role() == OPTreeItem::SUBPROP && role == Qt::EditRole )
    {
        item->setData( 1 , value ) ;
        QVariant v = item->parent()->data( 1 ) ;
        switch( v.type() )
        {
            break ; case QVariant::Rect :
            {
                //qDebug() << item->parent()->child( 0 )->data( 1 ).toInt() ;
                QRect r ( item->parent()->child( 0 )->data( 1 ).toInt()
                          ,item->parent()->child( 1 )->data( 1 ).toInt()
                          ,item->parent()->child( 2 )->data( 1 ).toInt()
                          ,item->parent()->child( 3 )->data( 1 ).toInt()) ;

                QVariantMap m ;

                m.insert( item->parent()->data( 0 ).toString() , r ) ;
                emit propsEdited ( m ) ;
                emit dataChanged (index, index);
            }
                //emit dataChanged ( this->index( item->parent( )
            break ; case QVariant::Size :
            {
                QSize s ( item->parent()->child( 0 )->data( 1 ).toInt()
                          ,item->parent()->child( 1 )->data( 1 ).toInt() ) ;
                QVariantMap m ;

                m.insert( item->parent()->data( 0 ).toString() , s ) ;
                emit propsEdited ( m ) ;
                emit dataChanged (index, index);
            }

            break ; case QVariant::Point :
            {
                QPoint p ( item->parent()->child( 0 )->data( 1 ).toInt()
                          ,item->parent()->child( 1 )->data( 1 ).toInt() ) ;
                QVariantMap m ;

                m.insert( item->parent()->data( 0 ).toString() , p ) ;
                emit propsEdited ( m ) ;
                emit dataChanged (index, index);
            }

            break ; case QVariant::Font :
            ;
            break ; default :
            ;
        }
    }

    else if ( ( item->data( 1 ).type() == value.type()
                || ( item->data( 1 ).type() == QVariant::Bool
                && value.type() == QVariant::Int ) )
              && item->role() == OPTreeItem::PROP
              && ( role == Qt::EditRole || role == Qt::CheckStateRole ) )
    {
        if ( item->data( 1 ).type() == QVariant::Bool )
            item->setData( 1 , value.toBool() ) ;
        else
            item->setData( 1 , value ) ;

        QVariantMap m ;

        m.insert( item->data( 0 ).toString() , item->data( 1 ) ) ;
        emit propsEdited ( m ) ;
        emit dataChanged (index, index);
        return true;
    }

    return false;
}
void QObPropModel::fontEdited( const QModelIndex & mIndex , QFont font )
{
    setData( mIndex , font ) ;
}
