#include "selectionsquares.h"

SelectionSquares::SelectionSquares ( QWidget * trg , QWidget *parent ) :
    QWidget ( parent ) , s ( 6 )
{
    squares << new SelectionSquare ( s , Qt::SizeFDiagCursor , 0 , 0 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeVerCursor , 0.5 , 0 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeBDiagCursor , 1 , 0 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeHorCursor , 1 , 0.5 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeFDiagCursor , 1 , 1 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeVerCursor , 0.5 , 1 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeBDiagCursor , 0 , 1 ,  this ) ;
    squares << new SelectionSquare ( s , Qt::SizeHorCursor , 0 , 0.5 ,  this ) ;

    foreach ( SelectionSquare * s , squares )
    {
        connect ( s , SIGNAL (moveSignal (QPoint)) , SLOT (squareMove (QPoint)) ) ;
        connect ( s , SIGNAL (endMoving ()) , SLOT (endMoveSlot ())) ;
        s->show () ;
    }

    attach ( trg ) ;
    update () ;
    setMouseTracking ( true ) ;
}

void SelectionSquares::attach ( QWidget * trg )
{
    target = trg ;
    disconnect () ;
    connect ( target , SIGNAL (destroyed ()) , SLOT (deleteLater ()) ) ;
    tuneGeometry () ;
    update () ;
}

void SelectionSquares::tuneGeometry ()
{
    if ( ! target )
        return ;
    setGeometry (target->frameGeometry ().adjusted ( - s / 2 , - s / 2 , s / 2 , s / 2 ) ) ;
    stackUnder ( target ) ;
    foreach ( SelectionSquare * s , squares )
        s->updatePos () ;
    show ();
}

QWidget * SelectionSquares::getTarget ()
{
    if ( target )
        return target.data () ;
    else
        return 0 ;
}

void SelectionSquares::squareMove ( QPoint p )
{
    int index = squares.indexOf (  static_cast< SelectionSquare * > ( sender () )  ) ;

    emit moveSignal ( p , index ) ;
}

void SelectionSquares::endMoveSlot ()
{
    emit endMoveSignal () ;
}

SelectionSquare::SelectionSquare ( qreal side , QCursor c , qreal xc , qreal yc , QWidget *parent ) :
    QWidget ( parent ) , s ( side ) , cursor ( c ) , xCoef ( xc ) , yCoef ( yc ) , dragging ( false )
{
    setGeometry ( 0 , 0 , s , s ) ;
    setMouseTracking ( true ) ;
    updatePos () ;
    setCursor ( cursor ) ;
}

void SelectionSquare::paintEvent (QPaintEvent *)
{
    QPainter painter ( this ) ;
    painter.setPen ( QPen ( QBrush (  Qt::blue ) , 1 ) ) ;
    painter.setBrush ( QBrush (  Qt::blue ) ) ;
    painter.drawRect ( 0 , 0 , s , s ) ;
}

void SelectionSquare::mouseMoveEvent (QMouseEvent * e)
{
    if ( dragging )
    {
        emit moveSignal ( e->globalPos () - oldPos ) ;
        oldPos = e->globalPos () ;
    }
}

void SelectionSquare::mousePressEvent (QMouseEvent * )
{
    dragging = true ;
    oldPos = QCursor::pos () ;
}

void SelectionSquare::mouseReleaseEvent (QMouseEvent * )
{
    dragging = false ;
    emit endMoving () ;
}

void SelectionSquare::updatePos ()
{
    move (  ( qreal ( parentWidget ()->width () ) - s ) * xCoef ,  ( qreal ( parentWidget ()->height () ) - s ) * yCoef ) ;
}
