#include "wpalette.h"
#include "widgetfactory.h"
#include <QClipboard>
#include <QApplication>
#include <QVBoxLayout>
#include <QMetaProperty>
#include "constants.h"
#include "props_functions.h"
WidgetsPalette::WidgetsPalette (QWidget *parent , QString cat )
    : QFrame (parent)
{
    setSizePolicy ( QSizePolicy::Maximum , QSizePolicy::Preferred ) ;
    setLayout ( new QVBoxLayout ( this ) ) ;
    QList < QWidget * > list ;
    if ( cat == "" )
        list = WidgetFactory::instance ()->newAll () ;
    else
        list = WidgetFactory::instance ()->newByCategory( cat ) ;
    foreach ( QWidget * w , list )
    {
        layout ()->addWidget ( w ) ;
        w->installEventFilter ( this ) ;
    }

    static_cast< QVBoxLayout * >  ( layout () )->addStretch () ;
}

void WidgetsPalette::mousePressEvent (QMouseEvent *event)
{
    if ( event->button () != Qt::LeftButton )
        return ;
    QWidget * child = qobject_cast<QWidget*> (childAt (event->pos ()));
    if  (!child)
        return;
    while ( child->parentWidget () != this && child->parentWidget () != 0 )
        child = child->parentWidget () ;

    QPoint hotSpot = child->mapFromGlobal ( QCursor::pos () ) ;

    QMimeData *mimeData = new QMimeData;
    mimeData->setText (child->metaObject ()->className ());

    QPixmap pixmap (child->size ());
    child->render (&pixmap);

    QDrag *drag = new QDrag (this);
    mimeData->setData ("application/x-hotspot",
                      QByteArray::number (hotSpot.x ())
                      + " " + QByteArray::number (hotSpot.y ()));

    drag->setMimeData (mimeData);
    drag->setPixmap (pixmap);
    drag->setHotSpot (hotSpot);

    WidgetFactory::instance()->copyToCB ( child ) ;

    drag->exec (Qt::CopyAction , Qt::CopyAction);
}







bool WidgetsPalette::eventFilter ( QObject * watched, QEvent * event )
{
    if ( children ().contains ( watched ) && event->type () == QEvent::MouseButtonPress )
    {
        QMouseEvent * me = static_cast< QMouseEvent * > ( event ) ;
        QWidget * w = static_cast< QWidget * > ( watched ) ;

        if (  ( me->buttons () & Qt::LeftButton ) && w)
        {
            mousePressEvent ( new QMouseEvent ( me->type () ,
                                              me->pos () + w->pos () ,
                                              me->button () ,
                                              me->buttons () ,
                                              me->modifiers ()) ) ;
            return true ;
        }
    }
    return false ;
}
